@@ -199,6 +199,7 @@ RENDER_MODES = {
 
 
 def get_column_names(data, cfg, sep='/'):
+    """Retrieve columnn names for CSV rendering."""
     columns_order = ['resource', 'metric', 'aggregation', 'role']
     columns = []
     column_names = data.columns.names
@@ -215,6 +216,7 @@ def get_column_names(data, cfg, sep='/'):
 
 
 def df_to_csv(dataframe, cfg):
+    """Render a dataframe to a CSV string."""
     dataframe = _prepare_data_frame(dataframe, hierarchical=['resource', 'metric'])
     columns = get_column_names(dataframe, cfg)
     # Use ISO formatting with microsecond support
@@ -222,6 +224,7 @@ def df_to_csv(dataframe, cfg):
 
 
 def df(pandas_object, render_mode=RENDER_MODE_METRIC_FLAT_DICT, render_options=None):
+    """Render a dataframe to a json-serializable value."""
     if (render_options is not None and 'render_mode' in render_options):
         render_mode = render_options['render_mode']
     r_args = RENDER_MODES.get(render_mode, {}).get('options', {}).copy()
@@ -238,6 +241,7 @@ def df(pandas_object, render_mode=RENDER_MODE_METRIC_FLAT_DICT, render_options=N
 
 
 def df1(pandas_object, render_mode=RENDER_MODE_METRIC_FLAT_DICT, render_options=None):
+    """Render a dataframe to a json-serializable value."""
     r = df(pandas_object, render_mode, render_options)
     if isinstance(r, list):
         return r[0] if len(r) == 1 else {} if len(r) == 0 else r
@@ -245,6 +249,7 @@ def df1(pandas_object, render_mode=RENDER_MODE_METRIC_FLAT_DICT, render_options=
 
 
 def parse_df(data, render_mode=None, render_options=None):
+    """Parse a json-compatible data structure to a Dataframe."""
     if render_mode is None:
         if isinstance(data, dict) and 'data' in data:
             if 'rows' in data:
@@ -273,24 +278,33 @@ def _parse_df_header_column_array(data, data_axis=DATA_AXIS_ROW, **r_arg):
 
 
 def _parse_df_header_row_array(data, data_axis=DATA_AXIS_COLUMN, **r_arg):
-    if 'columns' not in data:
-        raise ValueError('cannot parse row-based data without `column` specification')
-    row_names, column_index = _parse_row_names_and_column_index(data['columns'])
-    data_values = data['data']
+
+    data_values = data.get('data', [])
+
     if data_axis == DATA_AXIS_COLUMN:
         dataframe = pd.DataFrame(data_values)
     else:
         dataframe = pd.DataFrame()
         for col_idx, data_series in enumerate(data_values):
             dataframe[col_idx] = data_series
-    if row_names[0] == 'timestamp':
-        idx = pd.DatetimeIndex(pd.to_datetime(dataframe.iloc[:, 0], unit='ms', utc=True))
-        idx.name = row_names[0]
-        dataframe = dataframe.iloc[:, len(row_names):].set_index(idx)
-    else:
-        dataframe = dataframe.set_index(list(range(len(row_names))))  # pylint: disable=no-member
-        dataframe.index.names = row_names
-    dataframe.columns = column_index
+
+    if 'columns' in data:
+        row_names, column_index = _parse_row_names_and_column_index(data['columns'])
+        if len(dataframe) == 0:
+            dataframe = pd.DataFrame(columns=column_index, index=pd.DatetimeIndex([], name='timestamp'))
+        else:
+            if row_names[0] == 'timestamp':
+                idx = pd.DatetimeIndex(
+                    pd.to_datetime(dataframe.iloc[:, 0], unit='ms', utc=True),
+                    name='timestamp'
+                )
+                dataframe = dataframe.iloc[:, len(row_names):].set_index(idx)
+                row_names = row_names[1:]
+            elif row_names:
+                dataframe = dataframe.set_index(list(range(len(row_names))))  # pylint: disable=no-member
+                dataframe.index.names = row_names
+            dataframe.columns = column_index
+
     if 'attributes' in data:
         dataframe.tsa.table_attributes.update(data['attributes'])
     return dataframe
@@ -326,14 +340,14 @@ def _keys_and_key_series(index, key, iso_timestamp, **render_options):
     if isinstance(index, pd.DatetimeIndex):
         key = index.name or 'timestamp'
         keys.append(key)
-        key_series.append(
-            list((index - pd.Timestamp('1970-01-01T00:00Z')) // pd.Timedelta('1ms'))
-        )
+        ts_values = list(
+            (index - pd.Timestamp('1970-01-01T00:00Z')) // pd.Timedelta('1ms')
+        ) if len(index) else []
+        key_series.append(ts_values)
         if iso_timestamp:
             keys.append(key+'_iso')
-            key_series.append(
-                list(index.strftime('%Y-%m-%dT%H:%M:%S,%f%z'))
-            )
+            iso_ts_values = list(index.strftime('%Y-%m-%dT%H:%M:%S,%f%z'))
+            key_series.append(iso_ts_values)
     elif isinstance(index, pd.MultiIndex):
         for lvl_idx, lvl_name in enumerate(index.names):
             keys.append(lvl_name)
@@ -413,8 +427,8 @@ def _df_header_row_array(
         column_key=None, iso_timestamp=True, data_axis=DATA_AXIS_COLUMN,
         **render_options
 ):
-    if pandas_object is None or len(pandas_object) == 0:
-        return {}
+    if pandas_object is None:
+        pandas_object = pd.DataFrame()
     data_frame = _prepare_data_frame(pandas_object, **render_options)
     column_renderer = _header_renderer(data_frame.columns, column_key, iso_timestamp)
     if data_axis == DATA_AXIS_COLUMN:
@@ -453,8 +467,8 @@ def _df_header_column_array(
         column_key='column', iso_timestamp=True,  data_axis=DATA_AXIS_ROW,
         **render_options
 ):
-    if pandas_object is None or len(pandas_object) == 0:
-        return {}
+    if pandas_object is None:
+        pandas_object = pd.DataFrame()
     data_frame = _prepare_data_frame(pandas_object, **render_options)
     row_renderer = _header_renderer(data_frame.index, row_key, iso_timestamp)
     if data_axis == DATA_AXIS_ROW:
@@ -563,14 +577,16 @@ def _render_key(k, key_separator='.', key_skip_empty=False, **kwargs):
     return str(k)
 
 
-def render(v, **render_options):
-    return _render_value(v, **render_options)
+def render(value, **render_options):
+    """Convert a value to a json-serializable one."""
+    return _render_value(value, **render_options)
 
 
-def render_timestamp(v, iso_timestamp=False, **render_options):
+def render_timestamp(value, iso_timestamp=False, **render_options):
+    """Render a timestamp."""
     if iso_timestamp:
-        return v.isoformat()
-    return int(v.timestamp() * 1000)
+        return value.isoformat()
+    return int(value.timestamp() * 1000)
 
 
 def _render_value(v, **render_options):
