@@ -1,5 +1,5 @@
 
-"""REST client for the Waylay Time Series Analytics Service"""
+"""REST client for the Waylay Time Series Analytics Service."""
 
 from .._base import WaylayService
 
@@ -8,9 +8,12 @@ from .about import AboutResource
 
 
 class ByomlService(WaylayService):
-    """REST client for the Waylay BYOML Service"""
+    """REST client for the Waylay BYOML Service."""
+
+    service_key = 'byoml'
     config_key = 'byoml'
     default_root_url = 'https://byoml.waylay.io'
+
     resource_definitions = {
         'model': ModelResource,
         'about': AboutResource
@@ -18,8 +21,3 @@ class ByomlService(WaylayService):
 
     model: ModelResource
     about: AboutResource
-
-    def __init__(self):
-        super().__init__(
-            json_encode_body=True
-        )
