@@ -1,4 +1,4 @@
-"""REST definitions for status of Waylay BYOML service"""
+"""REST definitions for status of Waylay BYOML service."""
 
 from .._base import WaylayResource
 from .._decorators import (
@@ -10,7 +10,8 @@ DEFAULT_DECORATORS = [exception_decorator, return_body_decorator]
 
 
 class AboutResource(WaylayResource):
-    """static status endpoint"""
+    """Static status endpoint."""
+
     actions = {
         'health': {
             'method': 'GET',
